# SPDX-FileCopyrightText: © 2021 Antonio López Rivera <antonlopezr99@gmail.com>
# SPDX-License-Identifier: GPL-3.0-only

"""
MATMOS: Atmospheric model library
---------------------------------
"""

from matmos.models.ISA import ISA
